@@ -18,6 +18,7 @@
  *
  * Authors:
  *   Chema Celorio <chema@celorio.com>
+ *   Tristan Van Berkom <tvb@gnome.org>
  */
 
 #ifdef HAVE_CONFIG_H
@@ -550,7 +551,7 @@ glade_popup_create_clipboard_menu (GladeWidget *widget)
 	
 	popup_menu = gtk_menu_new ();
 
-	if (GTK_WIDGET_TOPLEVEL (glade_widget_get_object (widget)))
+	if (gtk_widget_is_toplevel (GTK_WIDGET (glade_widget_get_object (widget))))
 	{
 		glade_popup_append_item (popup_menu, GTK_STOCK_PASTE, 
 					 NULL, NULL, TRUE,
@@ -797,3 +798,15 @@ glade_popup_simple_pop (GdkEventButton *event)
 		NULL, NULL, button, event_time);
 }
 
+gboolean
+glade_popup_is_popup_event (GdkEventButton *event)
+{
+	g_return_val_if_fail (event, FALSE);
+
+#ifdef MAC_INTEGRATION
+	return (event->type == GDK_BUTTON_PRESS && event->button == 1 && ((event->state & GDK_MOD1_MASK) != 0));
+#else
+	return (event->type == GDK_BUTTON_PRESS && event->button == 3);
+#endif
+}
+
