@@ -518,7 +518,8 @@ glade_eprop_accel_show_dialog (GtkWidget           *dialog_button,
 
 	gtk_container_set_border_width (GTK_CONTAINER (vbox), 6);
 
-	gtk_box_pack_start (GTK_BOX (GTK_DIALOG (dialog)->vbox), vbox, TRUE, TRUE, 0);
+	gtk_box_pack_start (GTK_BOX (gtk_dialog_get_content_area (GTK_DIALOG (dialog))),
+			    vbox, TRUE, TRUE, 0);
 
 	sw = gtk_scrolled_window_new (NULL, NULL);
 	gtk_widget_show (sw);
@@ -586,7 +587,7 @@ glade_eprop_accel_create_input (GladeEditorProperty *eprop)
 
 	hbox               = gtk_hbox_new (FALSE, 0);
 	eprop_accel->entry = gtk_entry_new ();
-	gtk_entry_set_editable (GTK_ENTRY (eprop_accel->entry), FALSE);
+	gtk_editable_set_editable (GTK_EDITABLE (eprop_accel->entry), FALSE);
 	gtk_widget_show (eprop_accel->entry);
 	gtk_box_pack_start (GTK_BOX (hbox), eprop_accel->entry, TRUE, TRUE, 0);
 
